/*
 * OS/2 VT220 Emulation.
 *
 * Copyright (c) 1995-1997 Robert Muchsel, muchsel@acm.org
 * (Licence amended 2000-06-16 by Rick Moen, rick@linuxmafia.com, with 
 * author's permission, to fix a licence conflict.)
 *
 * Permission is granted to use this code under the GNU GPL,
 * with the additional permission that it may be linked against
 * Ylnen's SSH version 1.2.13 source code or The OpenBSD Project's
 * OpenSSH source code, without the GPL applying to the latter two 
 * codebases.  This copyright notice must not be removed.
 *
 ******************************************************************************
 *                  GNU GENERAL PUBLIC LICENSE
 *                     Version 2, June 1991
 *
 *  Copyright (C) 1989, 1991 Free Software Foundation, Inc.
 *                           675 Mass Ave, Cambridge, MA 02139, USA
 *  Everyone is permitted to copy and distribute verbatim copies
 *  of this license document, but changing it is not allowed.
 *
 *                          Preamble
 *
 *   The licenses for most software are designed to take away your
 * freedom to share and change it.  By contrast, the GNU General Public
 * License is intended to guarantee your freedom to share and change free
 * software--to make sure the software is free for all its users.  This
 * General Public License applies to most of the Free Software
 * Foundation's software and to any other program whose authors commit to
 * using it.  (Some other Free Software Foundation software is covered by
 * the GNU Library General Public License instead.)  You can apply it to
 * your programs, too.
 *
 *   When we speak of free software, we are referring to freedom, not
 * price.  Our General Public Licenses are designed to make sure that you
 * have the freedom to distribute copies of free software (and charge for
 * this service if you wish), that you receive source code or can get it
 * if you want it, that you can change the software or use pieces of it
 * in new free programs; and that you know you can do these things.
 *
 *   To protect your rights, we need to make restrictions that forbid
 * anyone to deny you these rights or to ask you to surrender the rights.
 * These restrictions translate to certain responsibilities for you if you
 * distribute copies of the software, or if you modify it.
 *
 *   For example, if you distribute copies of such a program, whether
 * gratis or for a fee, you must give the recipients all the rights that
 * you have.  You must make sure that they, too, receive or can get the
 * source code.  And you must show them these terms so they know their
 * rights.
 *
 *   We protect your rights with two steps: (1) copyright the software, and
 * (2) offer you this license which gives you legal permission to copy,
 * distribute and/or modify the software.
 *
 *   Also, for each author's protection and ours, we want to make certain
 * that everyone understands that there is no warranty for this free
 * software.  If the software is modified by someone else and passed on, we
 * want its recipients to know that what they have is not the original, so
 * that any problems introduced by others will not reflect on the original
 * authors' reputations.
 *
 *   Finally, any free program is threatened constantly by software
 * patents.  We wish to avoid the danger that redistributors of a free
 * program will individually obtain patent licenses, in effect making the
 * program proprietary.  To prevent this, we have made it clear that any
 * patent must be licensed for everyone's free use or not licensed at all.
 *
 *   The precise terms and conditions for copying, distribution and
 * modification follow.
 *
 *                  GNU GENERAL PUBLIC LICENSE
 *    TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION
 *
 *   0. This License applies to any program or other work which contains
 * a notice placed by the copyright holder saying it may be distributed
 * under the terms of this General Public License.  The "Program", below,
 * refers to any such program or work, and a "work based on the Program"
 * means either the Program or any derivative work under copyright law:
 * that is to say, a work containing the Program or a portion of it,
 * either verbatim or with modifications and/or translated into another
 * language.  (Hereinafter, translation is included without limitation in
 * the term "modification".)  Each licensee is addressed as "you".
 *
 * Activities other than copying, distribution and modification are not
 * covered by this License; they are outside its scope.  The act of
 * running the Program is not restricted, and the output from the Program
 * is covered only if its contents constitute a work based on the
 * Program (independent of having been made by running the Program).
 * Whether that is true depends on what the Program does.
 *
 *   1. You may copy and distribute verbatim copies of the Program's
 * source code as you receive it, in any medium, provided that you
 * conspicuously and appropriately publish on each copy an appropriate
 * copyright notice and disclaimer of warranty; keep intact all the
 * notices that refer to this License and to the absence of any warranty;
 * and give any other recipients of the Program a copy of this License
 * along with the Program.
 *
 * You may charge a fee for the physical act of transferring a copy, and
 * you may at your option offer warranty protection in exchange for a fee.
 *
 *   2. You may modify your copy or copies of the Program or any portion
 * of it, thus forming a work based on the Program, and copy and
 * distribute such modifications or work under the terms of Section 1
 * above, provided that you also meet all of these conditions:
 *
 *     a) You must cause the modified files to carry prominent notices
 *     stating that you changed the files and the date of any change.
 *
 *     b) You must cause any work that you distribute or publish, that in
 *     whole or in part contains or is derived from the Program or any
 *     part thereof, to be licensed as a whole at no charge to all third
 *     parties under the terms of this License.
 *
 *     c) If the modified program normally reads commands interactively
 *     when run, you must cause it, when started running for such
 *     interactive use in the most ordinary way, to print or display an
 *     announcement including an appropriate copyright notice and a
 *     notice that there is no warranty (or else, saying that you provide
 *     a warranty) and that users may redistribute the program under
 *     these conditions, and telling the user how to view a copy of this
 *     License.  (Exception: if the Program itself is interactive but
 *     does not normally print such an announcement, your work based on
 *     the Program is not required to print an announcement.)
 *
 * These requirements apply to the modified work as a whole.  If
 * identifiable sections of that work are not derived from the Program,
 * and can be reasonably considered independent and separate works in
 * themselves, then this License, and its terms, do not apply to those
 * sections when you distribute them as separate works.  But when you
 * distribute the same sections as part of a whole which is a work based
 * on the Program, the distribution of the whole must be on the terms of
 * this License, whose permissions for other licensees extend to the
 * entire whole, and thus to each and every part regardless of who wrote it.
 *
 * Thus, it is not the intent of this section to claim rights or contest
 * your rights to work written entirely by you; rather, the intent is to
 * exercise the right to control the distribution of derivative or
 * collective works based on the Program.
 *
 * In addition, mere aggregation of another work not based on the Program
 * with the Program (or with a work based on the Program) on a volume of
 * a storage or distribution medium does not bring the other work under
 * the scope of this License.
 *
 *   3. You may copy and distribute the Program (or a work based on it,
 * under Section 2) in object code or executable form under the terms of
 * Sections 1 and 2 above provided that you also do one of the following:
 *
 *     a) Accompany it with the complete corresponding machine-readable
 *     source code, which must be distributed under the terms of Sections
 *     1 and 2 above on a medium customarily used for software interchange; or,
 *
 *     b) Accompany it with a written offer, valid for at least three
 *     years, to give any third party, for a charge no more than your
 *     cost of physically performing source distribution, a complete
 *     machine-readable copy of the corresponding source code, to be
 *     distributed under the terms of Sections 1 and 2 above on a medium
 *     customarily used for software interchange; or,
 *
 *     c) Accompany it with the information you received as to the offer
 *     to distribute corresponding source code.  (This alternative is
 *     allowed only for noncommercial distribution and only if you
 *     received the program in object code or executable form with such
 *     an offer, in accord with Subsection b above.)
 *
 * The source code for a work means the preferred form of the work for
 * making modifications to it.  For an executable work, complete source
 * code means all the source code for all modules it contains, plus any
 * associated interface definition files, plus the scripts used to
 * control compilation and installation of the executable.  However, as a
 * special exception, the source code distributed need not include
 * anything that is normally distributed (in either source or binary
 * form) with the major components (compiler, kernel, and so on) of the
 * operating system on which the executable runs, unless that component
 * itself accompanies the executable.
 *
 * If distribution of executable or object code is made by offering
 * access to copy from a designated place, then offering equivalent
 * access to copy the source code from the same place counts as
 * distribution of the source code, even though third parties are not
 * compelled to copy the source along with the object code.
 *
 *   4. You may not copy, modify, sublicense, or distribute the Program
 * except as expressly provided under this License.  Any attempt
 * otherwise to copy, modify, sublicense or distribute the Program is
 * void, and will automatically terminate your rights under this License.
 * However, parties who have received copies, or rights, from you under
 * this License will not have their licenses terminated so long as such
 * parties remain in full compliance.
 *
 *   5. You are not required to accept this License, since you have not
 * signed it.  However, nothing else grants you permission to modify or
 * distribute the Program or its derivative works.  These actions are
 * prohibited by law if you do not accept this License.  Therefore, by
 * modifying or distributing the Program (or any work based on the
 * Program), you indicate your acceptance of this License to do so, and
 * all its terms and conditions for copying, distributing or modifying
 * the Program or works based on it.
 *
 *   6. Each time you redistribute the Program (or any work based on the
 * Program), the recipient automatically receives a license from the
 * original licensor to copy, distribute or modify the Program subject to
 * these terms and conditions.  You may not impose any further
 * restrictions on the recipients' exercise of the rights granted herein.
 * You are not responsible for enforcing compliance by third parties to
 * this License.
 *
 *   7. If, as a consequence of a court judgment or allegation of patent
 * infringement or for any other reason (not limited to patent issues),
 * conditions are imposed on you (whether by court order, agreement or
 * otherwise) that contradict the conditions of this License, they do not
 * excuse you from the conditions of this License.  If you cannot
 * distribute so as to satisfy simultaneously your obligations under this
 * License and any other pertinent obligations, then as a consequence you
 * may not distribute the Program at all.  For example, if a patent
 * license would not permit royalty-free redistribution of the Program by
 * all those who receive copies directly or indirectly through you, then
 * the only way you could satisfy both it and this License would be to
 * refrain entirely from distribution of the Program.
 *
 * If any portion of this section is held invalid or unenforceable under
 * any particular circumstance, the balance of the section is intended to
 * apply and the section as a whole is intended to apply in other
 * circumstances.
 *
 * It is not the purpose of this section to induce you to infringe any
 * patents or other property right claims or to contest validity of any
 * such claims; this section has the sole purpose of protecting the
 * integrity of the free software distribution system, which is
 * implemented by public license practices.  Many people have made
 * generous contributions to the wide range of software distributed
 * through that system in reliance on consistent application of that
 * system; it is up to the author/donor to decide if he or she is willing
 * to distribute software through any other system and a licensee cannot
 * impose that choice.
 *
 * This section is intended to make thoroughly clear what is believed to
 * be a consequence of the rest of this License.
 *
 *   8. If the distribution and/or use of the Program is restricted in
 * certain countries either by patents or by copyrighted interfaces, the
 * original copyright holder who places the Program under this License
 * may add an explicit geographical distribution limitation excluding
 * those countries, so that distribution is permitted only in or among
 * countries not thus excluded.  In such case, this License incorporates
 * the limitation as if written in the body of this License.
 *
 *   9. The Free Software Foundation may publish revised and/or new versions
 * of the General Public License from time to time.  Such new versions will
 * be similar in spirit to the present version, but may differ in detail to
 * address new problems or concerns.
 *
 * Each version is given a distinguishing version number.  If the Program
 * specifies a version number of this License which applies to it and "any
 * later version", you have the option of following the terms and conditions
 * either of that version or of any later version published by the Free
 * Software Foundation.  If the Program does not specify a version number of
 * this License, you may choose any version ever published by the Free Software
 * Foundation.
 *
 *   10. If you wish to incorporate parts of the Program into other free
 * programs whose distribution conditions are different, write to the author
 * to ask for permission.  For software which is copyrighted by the Free
 * Software Foundation, write to the Free Software Foundation; we sometimes
 * make exceptions for this.  Our decision will be guided by the two goals
 * of preserving the free status of all derivatives of our free software and
 * of promoting the sharing and reuse of software generally.
 *
 *                          NO WARRANTY
 *
 *   11. BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
 * FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN
 * OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
 * PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
 * OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS
 * TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE
 * PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
 * REPAIR OR CORRECTION.
 *
 *   12. IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
 * WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
 * REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES,
 * INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING
 * OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED
 * TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY
 * YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER
 * PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 *                   END OF TERMS AND CONDITIONS
 *
 *      Appendix: How to Apply These Terms to Your New Programs
 *
 *   If you develop a new program, and you want it to be of the greatest
 * possible use to the public, the best way to achieve this is to make it
 * free software which everyone can redistribute and change under these terms.
 *
 *   To do so, attach the following notices to the program.  It is safest
 * to attach them to the start of each source file to most effectively
 * convey the exclusion of warranty; and each file should have at least
 * the "copyright" line and a pointer to where the full notice is found.
 *
 *     <one line to give the program's name and a brief idea of what it does.>
 *     Copyright (C) 19yy  <name of author>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Also add information on how to contact you by electronic and paper mail.
 *
 * If the program is interactive, make it output a short notice like this
 * when it starts in an interactive mode:
 *
 *     Gnomovision version 69, Copyright (C) 19yy name of author
 *     Gnomovision comes with ABSOLUTELY NO WARRANTY; for details type `show w'.
 *     This is free software, and you are welcome to redistribute it
 *     under certain conditions; type `show c' for details.
 *
 * The hypothetical commands `show w' and `show c' should show the appropriate
 * parts of the General Public License.  Of course, the commands you use may
 * be called something other than `show w' and `show c'; they could even be
 * mouse-clicks or menu items--whatever suits your program.
 *
 * You should also get your employer (if you work as a programmer) or your
 * school, if any, to sign a "copyright disclaimer" for the program, if
 * necessary.  Here is a sample; alter the names:
 *
 *   Yoyodyne, Inc., hereby disclaims all copyright interest in the program
 *   `Gnomovision' (which makes passes at compilers) written by James Hacker.
 *
 *   <signature of Ty Coon>, 1 April 1989
 *   Ty Coon, President of Vice
 *
 * This General Public License does not permit incorporating your program into
 * proprietary programs.  If your program is a subroutine library, you may
 * consider it more useful to permit linking proprietary applications with the
 * library.  If this is what you want to do, use the GNU Library General
 * Public License instead of this License.
 ******************************************************************************
 *
 *
 * NO WARRANTY!
 *
 * This code does NOT provide a complete VT 220 emulation. However, it
 * should understand all control sequences and key codes commonly found in
 * the /etc/termcap files, especially the Linux "console". The emulation
 * has been checked using the vttest utility, the ncurses samples and
 * the ANSI Standard (X3.64).
 *
 * Not yet implemented (among others):
 *
 *   ESC =    DECKPAM - keypad application mode
 *   ESC >    DECKPNM - keypad numeric mode
 *
 *   VT52 emulation
 *
 *   All sequences returning messages (<ENQ>, CPR, DSR, DA, DECREQTPARM, etc.):
 *
 *   ESC [R
 *   ESC [0c, Z
 *   ESC [0n, [3n, [5n, [6n
 *
 * The emulation has no printing support. This is a _feature_ and avoids
 * unwanted screen dumps. Use OS/2 PrtSc or Copy/Paste instead.
 *
 * The code has been tested against Linux, Sun and SGI termcaps/terminfos.
 *
 * History:
 *
 * 1995/09/09  Initial version
 * 1995/09/10  Made some changes so it passes the vttest
 * 1995/09/11  Added some Linux console ESC sequences
 * 1996/01/01  Added null character mapping (Linux), Linux key codes
 * 1996/05/11  Added vt220_write() return value, merged viogetmode,
 *             vt220_enabled
 * 1997/01/04  some ANSI X3.64 additions, "double" width/height
 *             132 column support, fixed bug in ESC P recognition
 * 1997/03/11  Added CtrlSpace -> NULL, save cursor position, crlf is off
 *             by default
 * 1997/04/11  Accept screen heights other than 25
 *             Ignore NUL character
 *
 *
 */

/*
  DEBUG_VT220 prints all ESC codes and characters to stderr.
  Use ssh 2> logfile  (when using CMD)
   or ssh >&> logfile (when using 4OS2)
  to redirect the debug output.
 */
/* #define DEBUG_VT220 */

/*
  VERBOSE_VT220 prints a message if an unsupported ESC sequence is found
*/
#define VERBOSE_VT220

#ifdef VERBOSE_VT220
#define SHOW_ESC(x) fprintf(stderr,"%s\n", x)
#else
#define SHOW_ESC(x)
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#define INCL_VIO
#include <os2.h>
#include <sys/kbdscan.h>
#include "vt220.h"

/*
 * The ASCII control characters
 */
#define NUL   0
#define SOH   1
#define STX   2
#define ENQ   5
#define BEL   7
#define BS    8
#define HT    9
#define LF   10
#define VT   11
#define FF   12
#define CR   13
#define SO   14
#define SI   15
#define DLE  16
#define XON  17
#define XOFF 18
#define SYN  22
#define CAN  24
#define SUB  26
#define ESC  27
#define XFS  28
#define XGS  29
#define XRS  30
#define US   31
#define SP   32


/*
 * Colors
 */
#define COLOR_BLACK         0
#define COLOR_BLUE          1
#define COLOR_GREEN         2
#define COLOR_CYAN          3
#define COLOR_RED           4
#define COLOR_MAGENTA       5
#define COLOR_YELLOW        6
#define COLOR_WHITE         7
#define COLOR_BRIGHT        8

/*
 * Public variables
 */
int vt220_enabled = 1;


/*
 * Emulator state variables
 */
static int  xpos  = 1;
static int  ypos  = 1;
static int  topmargin = 1;
static int  botmargin = 25;

static int  state = 0;
static char esc_buffer[255];

#define MAXW 400  /* This is the maximum screen width, 400 should suffice */
                  /* The maximum screen size is MAXW*MAXR */
static char tabs[MAXW];

#define MAXR 256  /* Maximum screen height */
static char dblw[MAXR]; /* double width lines */

static int  screen_width;
static int  screen_height;

#define ATTR_DEFAULT   COLOR_WHITE
#define ATTR_UNDERLINE COLOR_GREEN+COLOR_BRIGHT
static int  attr           = ATTR_DEFAULT;
static int  attr_default   = ATTR_DEFAULT;
static int  attr_underline = ATTR_UNDERLINE;

/*
 * Color mapping
 */
static int  colors[8] = {
                          COLOR_BLACK,
                          COLOR_RED,
                          COLOR_GREEN,
                          COLOR_YELLOW,
                          COLOR_BLUE,
                          COLOR_MAGENTA,
                          COLOR_CYAN,
                          COLOR_WHITE
                        };

/*
 * VT100 graphics character mapping
 */
static int  vt_graph[] = {
  0x04, 0xB0, 0x1A, 0x17, 0x1B, 0x19, 0xF8, 0xF1, 0x15, 0x12,
  0xD9, 0xBF, 0xDA, 0xC0, 0xC5, 0xC4, 0xC4, 0xC4, 0xC4, 0xC4,
  0xC3, 0xB4, 0xC1, 0xC2, 0xB3, 0xF3, 0xF2, 0xE3, 0x9D, 0x9C,
  0xFA
};

static int  blink  = 0;
static int  bold   = 0;
static int  invis  = 0;
static int  underl = 0;
static int  revers = 0;

static int  saved_x      = 1;
static int  saved_y      = 1;
static int  saved_attr   = ATTR_DEFAULT;
static int  saved_blink  = 0;
static int  saved_bold   = 0;
static int  saved_invis  = 0;
static int  saved_underl = 0;
static int  saved_revers = 0;
static int  saved_g0     = 0;
static int  saved_g1     = 1;
static int  saved_chrset = 0;

static int  auto_wrap   = 1;
static int  insert_mode = 0;
static int  appl_mode   = 0;
static int  cursor_vis  = 1;
static int  col80       = 0;
static int  scr_reverse = 0;
static int  relative    = 0;
static int  crlf        = 1;
static int  graph_g0    = 0;
static int  graph_g1    = 1;
static int  charset     = 0;

VIOCURSORINFO Cursor;

/*
 * You must NOT remove the following notice. It will be compiled into
 * your executable file (if the linker is not smart enough to remove it...)
 */
char vt220_copyright[] = "Portions Copyright (c) 1995-1997 Robert Muchsel "\
                         "<muchsel@acm.org>\r\n";


static void set_screen_width(int);

/*
 * Convention: All exported functions are called vt220_...
 */


/*
 * Set default values (don't move the cursor)
 */
void defaults(void)
{
  int i;

  state = 0;
  xpos  = 1;
  ypos  = 1;

  memset(tabs, 0, sizeof(tabs));
  for (i = 1; i < MAXW; i += 8)
    tabs[i] = 1;
  memset(dblw, 0, sizeof(dblw));

  attr           = ATTR_DEFAULT;
  attr_default   = ATTR_DEFAULT;
  attr_underline = ATTR_UNDERLINE;

  blink  =
  bold   =
  invis  =
  underl =
  revers = 0;

  auto_wrap   = 1;
  insert_mode = 0;
  appl_mode   = 0;
  cursor_vis  = 1;
  col80       = 0;
  scr_reverse = 0;
  relative    = 0;
  crlf        = 1;
  graph_g0    = 0;
  graph_g1    = 1;
  charset     = 0;

  topmargin = 1;
  botmargin = screen_height;
}


/*
 * Reset the emulator (call this function at startup)
 */
void vt220_reset(void)
{
  int i;
  int x, y;

  VioGetCurPos((PUSHORT) &ypos, (PUSHORT) &xpos, 0);
  xpos++; /* OS/2 Vio coordinates are 0-based */
  ypos++;
  x = xpos;
  y = ypos;

  vt220_viogetmode(&screen_width, &screen_height, &i, &i);
  defaults();
  xpos = x;
  ypos = y;

  /* Save cursor */
  VioGetCurType(&Cursor, 0);
}


/*
 * Beep
 */
void beep(void)
{
  DosBeep(1660, 25);
}


/*
 * Disable (hide) cursor
 */
void cursor_off(void)
{
  VIOCURSORINFO CursorData;

  CursorData.yStart =  0;
  CursorData.cEnd   =  0;
  CursorData.cx     =  0;
  CursorData.attr   = -1;
  VioSetCurType(&CursorData, 0);        /* Hide cursor */
}

/*
 * Enable (show) cursor
 */
void cursor_on(void)
{
  VioSetCurType(&Cursor, 0);
}


/*
 * Move cursor, clip. Clip to margins if 'rel_y' is true
 */
void inline moveto(int rel_y, int x, int y)
{
  if (x < 1)
    x = 1;
  if (x > screen_width)
    x = screen_width;
  if (rel_y)
    {
      if (y < topmargin)
        y = topmargin;
      if (y > botmargin)
        y = botmargin;
    }
  else
    {
      if (y < 1)
        y = 1;
      if (y > screen_height)
        y = screen_height;
    }

  if (dblw[ypos] && (x*2 > screen_width))
    x = screen_width/2;

  xpos = x;
  ypos = y;

  if (cursor_vis)
    if (!dblw[ypos])
      VioSetCurPos(y-1, x-1, 0);
    else
      VioSetCurPos(y-1, (x-1)*2, 0);

  col80 = 0;
}


/*
 * Clear to end of line
 */
void clear_eol(void)
{
  char Cell[2];

  Cell[0] = SP;
  Cell[1] = attr_default;
  if (!dblw[ypos])
    VioWrtNCell(Cell, screen_width+1 - xpos, ypos-1, xpos-1, 0);
  else
    VioWrtNCell(Cell, (screen_width+1 - xpos)*2, ypos-1, (xpos-1)*2, 0);
}


/*
 * Clear to beginning of line
 */
void clear_bol(void)
{
  char Cell[2];

  Cell[0] = SP;
  Cell[1] = attr_default;
  if (!dblw[ypos])
    VioWrtNCell(Cell, xpos, ypos-1, 0, 0);
  else
    VioWrtNCell(Cell, xpos*2, ypos-1, 0, 0);
}


/*
 * Clear line
 */
void clear_line(void)
{
  char Cell[2];

  Cell[0] = SP;
  Cell[1] = attr_default;
  VioWrtNCell(Cell, screen_width, ypos-1, 0, 0);
}


/*
 * Clear to end of screen
 */
void clear_eos(void)
{
  int i;
  char Cell[2];

  Cell[0] = SP;
  Cell[1] = attr_default;
  VioWrtNCell(Cell, screen_width*screen_height - (ypos-1)*screen_width - (xpos-1),
              ypos-1, xpos-1, 0);

  for (i = ypos; i <= screen_height; i++)
    dblw[i] = 0;
}


/*
 * Clear to beginning of screen
 */
void clear_bos(void)
{
  int i;
  char Cell[2];

  Cell[0] = SP;
  Cell[1] = attr_default;
  VioWrtNCell(Cell, (ypos-1)*screen_width + xpos, 0, 0, 0);

  for (i = 1; i <= ypos; i++)
    dblw[i] = 0;
}


/*
 * Clear screen
 */
void clear_scr(char c)
{
  char Cell[2];

  Cell[0] = c;
  Cell[1] = attr_default;
  VioWrtNCell(Cell, screen_width*screen_height, 0, 0, 0);

  memset(dblw, 0, sizeof(dblw));
}


/*
 * Flip a color.
 */
int inline flip(int color)
{
  return (color & 0x88) |
        ((color & 0x70) >> 4) |
        ((color & 0x07) << 4);
}


/*
 * Reverse screen
 */
void reverse_scr()
{
  char Cell[2 * MAXW*MAXW];
  int  Length;

  /* Flip all colors */
  attr           = flip(attr);
  attr_default   = flip(attr_default);
  attr_underline = flip(attr_underline);

  /* Flip the screen */
  Length = 2 * screen_width*screen_height;
  VioReadCellStr(Cell, (PUSHORT) &Length, 0, 0, 0);
  for (Length = 0; Length < screen_width*screen_height; Length++)
    Cell[2*Length+1] = flip(Cell[2*Length+1]);
  VioWrtCellStr(Cell, 2*Length, 0, 0, 0);
}


/*
 * Scroll (direction -1 = UP, +1 = DOWN)
 */
void scroll(int direction, int topy, int boty, int numlines)
{
  int i;
  char Cell[2];

  Cell[0] = SP;
  Cell[1] = attr_default;
  if (direction < 0) {
    VioScrollUp(topy-1, 0, boty-1, screen_width-1, numlines, Cell, 0);
    for (i = topy; i <= boty - numlines; i++)
      dblw[i] = dblw[i + numlines];
    for (i = boty - numlines + 1; i <= boty; i++)
      dblw[i] = 0;
  }
  else {
    VioScrollDn(topy-1, 0, boty-1, screen_width-1, numlines, Cell, 0);
    for (i = boty; i >= topy + numlines; i--)
      dblw[i] = dblw[i - numlines];
    for (i = topy + numlines - 1; i >= boty; i--)
      dblw[i] = 0;
  }
}


/*
 * Write character to screen on 'xpos'/'ypos' using 'attr'
 * Take care of the rightmost column (col80) and autowrap
 */
void screen_write(unsigned char c)
{
  char Cell[2];
  int wrap80 = 0;

  switch (c)
    {
      case LF: /* advance one line, scroll if needed */
               if (ypos < botmargin)
                 ypos++;
               else
                 scroll(-1, topmargin, botmargin, 1);
      break;

      case CR:
               xpos = 1;
      break;

      case BS:
               xpos--;
      break;

      default:
               if (col80 && (xpos == screen_width || (dblw[ypos] && xpos == screen_width/2)))
                 if (auto_wrap) /* wrap line if longer than screenwidth chars */
                   {
                     if (ypos < botmargin)
                       ypos++;
                     else
                       scroll(-1, topmargin, botmargin, 1);
                     xpos = 1;
                       moveto(0, xpos, ypos);
                   }
                 else
                   return; /* forget the character */

               if (invis)
                 c = SP;
               else
                 if ((c >= 96) && (c <= 126) &&
                    ((charset && graph_g1) || (!charset && graph_g0)))
                   c = vt_graph[c - 96];

               if (insert_mode) {
                 Cell[0] = c;
                 Cell[1] = attr;
                 if (!dblw[ypos])
                   VioScrollRt(ypos-1, xpos-1, ypos-1, screen_width-1, 1, Cell, 0);
                 else {
                   VioScrollRt(ypos-1, (xpos-1)*2, ypos-1, screen_width-1, 1, Cell, 0);
                   Cell[0] = SP;
                   VioScrollRt(ypos-1, xpos*2-1, ypos-1, screen_width-1, 1, Cell, 0);
                 }
               }
               else {
                 if (!dblw[ypos])
                   VioWrtCharStrAtt((PCH) &c, 1, ypos-1, xpos-1, (PBYTE) &attr, 0);
                 else {
                   VioWrtCharStrAtt((PCH) &c, 1, ypos-1, (xpos-1)*2, (PBYTE) &attr, 0);
                   c = SP;
                   VioWrtCharStrAtt((PCH) &c, 1, ypos-1, xpos*2-1, (PBYTE) &attr, 0);
                 }
               }

               xpos++;
               wrap80 = xpos > dblw[ypos] ? screen_width/2 : screen_width;

      break;
    } /* switch */

  moveto(0, xpos, ypos);
  col80 = wrap80;
}


/*
 * Write a string to screen on 'xpos'/'ypos' using 'attr'.
 * This function does not wrap nor process control characters.
 */
void inline screen_fastwrite(int len, char *str)
{
  int i;
  char buffer[MAXW];
  char Cell[2];
  unsigned char *s, *t;
  int xp, l;

  s = str;

  if (dblw[ypos]) {
    t = buffer;
    for (i = 0; i < len; i++) {
      *t = *s; t++; s++;
      *t = SP; t++;
    }
    l = len*2;
    s = str = buffer;
    xp = (xpos-1)*2;
  }
  else {
    xp = xpos-1;
    l = len;
  }

  /* process characters */
  while (*s)
    {
      if (invis)
        *s = SP;
      else
        if ((*s >= 96) && (*s <= 126) &&
          ((charset && graph_g1) || (!charset && graph_g0)))
          *s = vt_graph[*s - 96];

      s++;
    }

  if (insert_mode)
    {
      Cell[0] = SP;
      Cell[1] = attr_default;

      VioScrollRt(ypos-1, xp, ypos-1, screen_width-1, l, Cell, 0);
    }

  VioWrtCharStrAtt((PCH) str, l, ypos-1, xp, (PBYTE) &attr, 0);

  moveto(0,xpos+len,ypos);
}


/*
 * expand line (double width)
 */
void expand(void)
{
  int  i;
  char Cell[2 * MAXW];
  int  Length;

  Length = screen_width*2;

  VioReadCellStr(Cell, (PUSHORT) &Length, ypos-1, 0, 0);
  for (i = screen_width/2 - 1; i >= 0; i--) {
    Cell[i*4] = Cell[i*2];
    Cell[i*4 + 2] = SP;
  }
  VioWrtCellStr(Cell, Length, ypos-1, 0, 0);

  dblw[ypos] = 1;
  moveto(0, xpos, ypos);
}

/*
 * shrink line (single width)
 */
void shrink(void)
{
  int  i;
  char Cell[2 * MAXW];
  int  Length;

  Length = screen_width*2;

  VioReadCellStr(Cell, (PUSHORT) &Length, ypos-1, 0, 0);
  for (i = 0; i <= screen_width/2 - 1; i++)
    Cell[i*2] = Cell[i*4];
  for (i = screen_width/2; i < screen_width; i++)
    Cell[i*2] = SP;
  VioWrtCellStr(Cell, Length, ypos-1, 0, 0);

  dblw[ypos] = 0;
  moveto(0, xpos, ypos);
}


/*
 * Delete character on screen
 */
void screen_del(int numchars)
{
  char Cell[2];

  Cell[0] = SP;
  Cell[1] = attr_default;
  if (!dblw[ypos])
    VioScrollLf(ypos-1, xpos-1, ypos-1, screen_width-1, numchars, Cell, 0);
  else
    VioScrollLf(ypos-1, (xpos-1)*2, ypos-1, screen_width-1, numchars*2, Cell, 0);
}


/*
 * Insert character on screen
 */
void screen_ins(int numchars)
{
  char Cell[2];

  Cell[0] = SP;
  Cell[1] = attr_default;
  if (!dblw[ypos])
    VioScrollRt(ypos-1, xpos-1, ypos-1, screen_width-1, numchars, Cell, 0);
  else
    VioScrollRt(ypos-1, (xpos-1)*2, ypos-1, screen_width-1, numchars*2, Cell, 0);
}


/*
 * Overwrite character on screen
 */
void screen_over(int numchars)
{
  char Cell[2];

  Cell[0] = SP;
  Cell[1] = attr_default;
  if (!dblw[ypos])
    VioWrtNCell(Cell, numchars+1 - xpos, ypos-1, xpos-1, 0);
  else
    VioWrtNCell(Cell, (numchars+1)*2 - xpos*2, ypos-1, (xpos-1)*2, 0);
}


/*
 * Save cursor position and attributes
 */
void save_cursor(void)
{
  saved_x      = xpos;
  saved_y      = ypos;
  saved_attr   = attr;
  saved_blink  = blink;
  saved_bold   = bold;
  saved_invis  = invis;
  saved_underl = underl;
  saved_revers = revers;
  saved_g0     = graph_g0;
  saved_g1     = graph_g1;
  saved_chrset = charset;
}


/*
 * Restore cursor position and attributes
 */
void restore_cursor(void)
{
  xpos     = saved_x;
  ypos     = saved_y;
  attr     = saved_attr;
  blink    = saved_blink;
  bold     = saved_bold;
  invis    = saved_invis;
  underl   = saved_underl;
  revers   = saved_revers;
  graph_g0 = saved_g0;
  graph_g1 = saved_g1;
  charset  = saved_chrset;

  moveto(0, xpos, ypos);
}

/*
 * Change screen width
 */
void set_screen_width(int width)
{
  int i;

  vt220_viosetmode(width, screen_height);
  vt220_viogetmode(&screen_width, &screen_height, &i, &i);
}


/*
 * Handle ESC sequences
 */
void esc_sequence()
{
#ifdef DEBUG_VT220
  fprintf(stderr, "ESC: %.*s (%c)\n", state-1, &esc_buffer[1],
          esc_buffer[1]);
#endif

  esc_buffer[state] = 0;

  /* first character is action code */
  switch (esc_buffer[1])
    {
      case 'D': /* IND - index */
                if (ypos < botmargin)
                  moveto(0, xpos, ypos+1);
                else
                  scroll(-1, topmargin, botmargin, 1);
                col80 = 0;
      break;

      case 'E': /* NEL - next line */
                screen_write(CR);
                screen_write(LF);
      break;

      case 'G': /* ESA - end of selected area */
                SHOW_ESC("ESA");
      break;

      case 'H': /* HTS - horizontal tabulation set */
                tabs[xpos] = 1;
      break;

      case 'I': /* HTJ - horizontal tabulation with justification */
                SHOW_ESC("HTJ");
      break;

      case 'J': /* VTS - vertical tabulation set */
                SHOW_ESC("VTS");
      break;

      case 'K': /* PLD - partial line down */
                SHOW_ESC("PLD");
      break;

      case 'L': /* PLU - partial line up */
                SHOW_ESC("PLU");
      break;

      case 'M': /* RI - reverse index */
                if (ypos > topmargin)
                  moveto(0, xpos, ypos-1);
                else
                  scroll(1, topmargin, botmargin, 1);
                col80 = 0;
      break;

      case 'N': /* SS2 - single shift 2 (G2 set) */
      case 'O': /* SS3 - single shift 3 (G3 set) */
                SHOW_ESC("SS2/3");
      break;

      case 'Q': /* PU1 - private use 1 */
      case 'R': /* PU2 - private use 2 */
                SHOW_ESC("PU1/2");
      break;

      case 'S': /* STS - set transmit state */
                SHOW_ESC("STS");
      break;

      case 'T': /* CCH - cancel previous character */
                SHOW_ESC("CCH");
      break;

      case 'U': /* MW - message waiting */
                SHOW_ESC("MW");
      break;

      case 'V': /* SPA - start of protected area */
                SHOW_ESC("SPA");
      break;

      case 'W': /* EPA - end of protected area */
                SHOW_ESC("EPA");
      break;

      case 'a': /* INT - interrupt */
                SHOW_ESC("INT");
      break;

      case 'b': /* EMI - enable manual input */
                SHOW_ESC("EMI");
      break;

      case 'c': /* RIS - reset to initial state */
                defaults();
                clear_scr(SP);
                moveto(0,1,1);
      break;

      case '7': /* DECSC - save cursor */
                save_cursor();
      break;

      case '8': /* DECRC - restore cursor */
                restore_cursor();
      break;

      case '\\':/* DMI - disable manual input */
                /* ignored */
      break;

      case '#': /* ESC # */
                switch (esc_buffer[2])
                  {
                    case '5': /* DECSWL - single width line */
                              if (dblw[ypos])
                                shrink();
                    break;

                    case '3': /* DECDHL - double height line top half */
                    case '4': /* DECDHL - double height line bottom half */
                    case '6': /* DECDWL - double width line */
                              if (!dblw[ypos])
                                expand();
                    break;

                    case '8': /* DECALN - self test */
                              clear_scr('E');
                              topmargin = 1;
                              botmargin = screen_height;
                              moveto(0,1,1);
                    break;

                    default:
                    break;
                  }
      break;

      case '(': /* SCS - select GO character set */
      case ')': /* SCS - select G1 character set */
                switch (esc_buffer[2])
                  {
                    case '0': /* VT graphics */
                              if (esc_buffer[1] == '(')
                                graph_g0 = 1;
                              else
                                graph_g1 = 1;
                    break;

                    case 'A': /* UK ASCII */
                    case 'B': /* US ASCII */
                    case 'K': /* user mapping */
                    case 'U': /* null mapping */
                    case '1':
                    case '2':
                              if (esc_buffer[1] == '(')
                                graph_g0 = 0;
                              else
                                graph_g1 = 0;
                    break;

                    default:  /* ignore */
                    break;
                  }
      break;

      default:  /* just ignore */
      break;
    }
}



/*
 * Handle ESC CSI screen control sequences
 */
void csi_sequence()
{
  int arg[200];
  int na = 0;
  char s[20];
  int escpos, spos, i, private, set;

  /* the buffer contains the complete ESC sequence, ESC[..;..;.X */

  esc_buffer[state] = 0;

  /* are there arguments? */
  escpos  = 2;
  private = 0;

  while (escpos < state)
    {
      spos = 0;
      if (esc_buffer[escpos] == '?')
        {
          private = 1;
          escpos++;
        }

      while ((esc_buffer[escpos] >= '0') && (esc_buffer[escpos] <= '9'))
        {
          s[spos] = esc_buffer[escpos];
          spos++;
          escpos++;
        }
      s[spos] = 0;
      if (*s)
        {
          if (!private)
            arg[na] = atoi(s);
          else
            arg[na] = -1*atoi(s);
          na++;
        }
      escpos++;
    }

  if (na == 0)
    arg[0] = 1; /* default in many cases */
  if (na <= 1)
    arg[1] = 1;

#ifdef DEBUG_VT220
  fprintf(stderr, "ESC CSI: %.*s (%c)\n", state-1, &esc_buffer[1],
          esc_buffer[state-1]);
#endif

  /* last character contains action code */
  switch (esc_buffer[state-1])
    {
      case 'A': /* CUU - cursor up */
                if (arg[0] == 0)
                  arg[0] = 1;
                moveto(relative,xpos,ypos-arg[0]);
      break;

      case 'e': /* VPR - vertical position relative */
      case 'B': /* CUD - cursor down */
                if (arg[0] == 0)
                  arg[0] = 1;
                moveto(relative,xpos,ypos+arg[0]);
      break;

      case 'a': /* HPR - horizontal position relative */
      case 'C': /* CUF - cursor forward */
                if (arg[0] == 0)
                  arg[0] = 1;
                moveto(0,xpos+arg[0],ypos);
      break;

      case 'D': /* CUB - cursor backward */
                if (arg[0] == 0)
                  arg[0] = 1;
                moveto(0,xpos-arg[0],ypos);
      break;

      case 'E': /* CNL - cursor next line */
                if (arg[0] == 0)
                  arg[0] = 1;
                moveto(relative,1,ypos+arg[0]);
      break;

      case 'F': /* CPL - cursor preceding line */
                if (arg[0] == 0)
                  arg[0] = 1;
                moveto(relative,1,ypos-arg[0]);
      break;

      case '`': /* HPA - horizontal position absolute */
      case 'G': /* CHA - cursor horizontal absolute */
                moveto(0, arg[0], ypos);
      break;

      case 'f': /* HVP - horizontal and vertical position */
      case 'H': /* CUP - cursor position */
                ypos =  arg[0] + (relative ? topmargin-1 : 0);
                moveto(relative, arg[1], ypos);
      break;

      case 'I': /* CHT - cursor horizontal tab */
                SHOW_ESC("CHT");
      break;

      case 'J': /* ED - erase in display */
                if ((na == 0) || (arg[0] == 0))
                  clear_eos();
                else
                  if (arg[0] == 1)
                    clear_bos();
                  else
                    if (arg[0] == 2)
                      clear_scr(SP);
      break;

      case 'K': /* EL - erase in line */
                if ((na == 0) || (arg[0] == 0))
                  clear_eol();
                else
                  if (arg[0] == 1)
                    clear_bol();
                  else
                    if (arg[0] == 2)
                      clear_line();
      break;

      case 'L': /* IL - insert line */
                for (i = 1; i <= arg[0]; i++)
                  scroll(1, ypos, botmargin, 1);
      break;

      case 'M': /* DL - delete line */
                for (i = 1; i <= arg[0]; ++i)
                  scroll(-1, ypos, botmargin, 1);
      break;

      case 'N': /* EF - erase in field */
                SHOW_ESC("EF");
      break;

      case 'O': /* EA - erase in area */
                SHOW_ESC("EA");
      break;

      case 'P': /* DCH - delete character */
                if (arg[0] > screen_width + 1 - xpos)
                  arg[0] = screen_width + 1 - xpos;
                screen_del(arg[0]);
      break;

      case 'Q': /* SEM - select edit extend mode */
                SHOW_ESC("SEM");
      break;

      case 'S': /* SU - scroll up */
                SHOW_ESC("SU");
      break;

      case 'T': /* SD - scroll down */
                SHOW_ESC("SD");
      break;

      case 'U': /* NP - next page */
                SHOW_ESC("NP");
      break;

      case 'V': /* PP - preceding page */
                SHOW_ESC("PP");
      break;

      case 'W': /* CTC - cursor tab control */
                SHOW_ESC("CTC");
      break;

      case 'X': /* ECH - erase character */
                if (arg[0] > screen_width + 1 - xpos)
                  arg[0] = screen_width + 1 - xpos;
                screen_over(arg[0]);
      break;

      case 'Y': /* CVT - cursor vertical tab */
                SHOW_ESC("CVT");
      break;

      case 'Z': /* CBT - cursor backward tab */
                SHOW_ESC("CBT");
      break;

      case '@': /* ICH - insert character */
                if (arg[0] > screen_width + 1 - xpos)
                  arg[0] = screen_width + 1 - xpos;
                screen_ins(arg[0]);
      break;

      case 'b': /* REP - repeat char or control */
                SHOW_ESC("REP");
      break;

      case 'c': /* DA - device attributes */
                SHOW_ESC("DA");
      break;

      case 'd': /* VPA - vertical position absolute */
                moveto(0, xpos, arg[0]);
      break;

      case 'g': /* TBC - tabulation clear */
                if ((na == 0) || (arg[0] == 0))
                  tabs[xpos] = 0;
                else
                  if (arg[0] == 3) /* clear all */
                    memset(tabs, 0, sizeof(tabs));
      break;

      case 'i': /* MC - media copy */
                SHOW_ESC("MC");
      break;

      case 'h': /* SM - select mode */
      case 'l': /* RM - reset mode */
                set = esc_buffer[state-1] == 'h';

                for (i = 0; i < na; i++)
                  {
                    switch (arg[i])
                      {
                        case  -1: /* DECCKM - cursor keys application mode */
                                  appl_mode   = set;
                                  crlf        = !set;
                        break;

                        case  -2: /* DECANM - VT52 mode */
                                  SHOW_ESC("DECANM");
                        break;

                        case  -3: /* DECCOLM - 132 columns */
                                  clear_scr(SP);
                                  set_screen_width(set ? 132 : 80);
                        break;

                        case  -5: /* DECSCNM - reverse screen */
                                  if (scr_reverse ^ set)
                                    {
                                      scr_reverse = set;
                                      reverse_scr();
                                    }
                        break;

                        case  -6: /* DECOM   - relative origin */
                                  relative    = set;
                                  moveto(relative, 1, 1);
                        break;

                        case  -7: /* DECAWM  - auto wrap */
                                  auto_wrap   = set;
                        break;

                        case -25: /* DECTCEM - cursor on */
                                  if (set)
                                    {
                                      cursor_on();
                                      cursor_vis = 1;
                                      moveto(0, xpos, ypos);
                                    }
                                  else
                                    {
                                      cursor_off();
                                      cursor_vis = 0;
                                    }
                        break;

                        case   4: /* IRM - insertion/replacement mode */
                                  insert_mode = set;
                        break;

                        case  20: /* LNM - linefeed/newline mode */
                                  crlf        = set;
                        break;

                        default:  /* ignore all others */
                        break;
                      }
                  }
      break;

      case 'm': /* SGR - select graphic rendition */
                if (na == 0)
                  {
                    attr   = attr_default;

                    blink  =
                    bold   =
                    invis  =
                    underl =
                    revers = 0;
                  }
                else
                  for (i = 0; i < na; i++)
                    {
                      switch (arg[i])
                        {
                          case  0: /* reset */
                                   attr   = attr_default;

                                   blink  =
                                   bold   =
                                   invis  =
                                   underl =
                                   revers = 0;
                          break;

                          case 21: /* bold off */
                          case 22:
                          case  1: /* bold */
                          case  2:
                                   if (bold ^ (arg[i] < 20))
                                     {
                                       bold   = arg[i] < 20;
                                       attr  ^= 8;
                                     }
                          break;

                          case 24: /* underline off */
                          case  4: /* underline */
                                   if (underl ^ (arg[i] == 4))
                                     {
                                       underl = arg[i] == 4;
                                       if (underl)
                                         attr = attr_underline;
                                       else
                                         attr = attr_default;
                                       if (bold)
                                         attr ^= 8;
                                       if (blink)
                                         attr ^= 0x80;
                                       if (revers) /* swap colors */
                                         attr = flip(attr);
                                     }
                          break;

                          case 25: /* blink off */
                          case  5: /* blink */
                                   if (blink ^ (arg[i] == 5))
                                     {
                                       blink = arg[i] == 5;
                                       attr ^= 0x80;
                                     }
                          break;

                          case 27: /* reverse off */
                          case  7: /* reverse */
                                   if (revers ^ (arg[i] == 7))
                                     {
                                       revers = arg[i] == 7;
                                       attr   = flip(attr);
                                     }
                          break;

                          case 28: /* invisible off */
                          case  8: /* invisible */
                                   invis = arg[i] == 8;
                          break;

                          case 30:
                          case 31:
                          case 32:
                          case 33:
                          case 34:
                          case 35:
                          case 36:
                          case 37: /* foreground colors */
                                   if (!scr_reverse)
                                     {
                                       attr &= 0xF8;
                                       attr |= colors[arg[i] - 30];
                                     }
                                   else
                                     {
                                       attr &= 0x8F;
                                       attr |= colors[arg[i] - 30] << 4;
                                     }
                          break;

                          case 40:
                          case 41:
                          case 42:
                          case 43:
                          case 44:
                          case 45:
                          case 46:
                          case 47: /* background colors */
                                   if (!scr_reverse)
                                     {
                                       attr &= 0x8F;
                                       attr |= colors[arg[i] - 40] << 4;
                                     }
                                   else
                                     {
                                       attr &= 0xF8;
                                       attr |= colors[arg[i] - 40];
                                     }
                          break;

                          default: /* ignore */
                          break;
                        }
                    }
      break;

      case 'n': /* DSR - device status report */
                SHOW_ESC("DSR");
      break;

      case 'o': /* DAQ - define area qualification */
                SHOW_ESC("DAQ");
      break;

      case 'r': /* DECSTBM - set top and bottom margins */
                i = na >= 2 ? arg[1] : screen_height;
                if ((arg[0] >= 1) && (arg[0] < i) && (i <= screen_height))
                  {
                    topmargin = arg[0];
                    botmargin = i;
                  }
                moveto(relative, 1, 1);
      break;

      case 's': /* ??? - save cursor and attributes */
                save_cursor();
      break;

      case 'u': /* ??? - restore cursor and attributes */
                restore_cursor();
      break;

      default:  /* ignore */
      break;
    }
}


/*
 * Advance the source pointer, read from buffered esc sequence if
 * needed, else from the buffer
 */
void inline advance(char **pcptr, int *remembered_chars, char *buffer)
{
  if (*remembered_chars > 0)
    {
      (*remembered_chars)--;
      if (*remembered_chars > 0)
        (*pcptr)++;
      else
        *pcptr = buffer;
    }
  else
    (*pcptr)++;
}


/*
 * Write a screen sequence and process characters. If an escape sequence
 * is detected, set the state variable and remember the sequence. If the
 * sequence isn't finished, the sequence is re-parsed from the esc_buffer
 * when the function is called again.
 */
int vt220_write(char *buffer, int len)
{
  char *bptr, *cptr;
  int pos, x;
  int remembered_chars;
  char buf[MAXW];

  remembered_chars = state;
  if (remembered_chars > 0)
    cptr = esc_buffer;
  else
    cptr = buffer;

  pos = -remembered_chars;

  while (pos < len)
    {
      switch (*cptr)
        {
          case ESC :  /* ESC sequence */
restart_ESC:
                      state = 0;
                      esc_buffer[state] = *cptr;
                      state++;
restart_ESC1:
                      advance(&cptr, &remembered_chars, buffer);
                      pos++;
                      if (pos >= len)
                        return len;

                      esc_buffer[state] = *cptr;
                      state++;

                      switch (*cptr)
                        {
                          case CAN :  /* cancel sequence */
                          case SUB :
                          case BS :
                                      goto quit_ESC;
                          break;

                          case ESC :  /* restart sequence */
                                      goto restart_ESC;
                          break;

                          case '[' :  /* CSI - screen control sequence */
                                      advance(&cptr, &remembered_chars, buffer);
                                      pos++;
                                      if (pos >= len)
                                        return len;

                                      esc_buffer[state] = *cptr;
                                      state++;

                                      if (*cptr == ESC)
                                        goto restart_ESC;
                                      if ((*cptr == CAN) || (*cptr == SUB))
                                        goto quit_ESC;
                                      if (*cptr == BS)
                                        { state = 1; goto restart_ESC1; }

                                      while ((*cptr < '@') || (*cptr > '~'))
                                        {
                                          if (*cptr == ESC)
                                            goto restart_ESC;
                                          if ((*cptr == CAN) || (*cptr == SUB))
                                            goto quit_ESC;
                                          if (*cptr == BS)
                                            state -= 2;

                                          advance(&cptr, &remembered_chars, buffer);
                                          pos++;
                                          if (pos >= len)
                                            return len;

                                          esc_buffer[state] = *cptr;
                                          state++;
                                        }
                                      csi_sequence();
                          break;

                          case '_' :  /* APC - application program command */
                          case 'P' :  /* DCS - device control string */
                          case '^' :  /* PM - privacy message */
                          case ']' :  /* OSC - operating system command */
                                      /* eat string until ST - string terminator (ESC \) */
                                      while (1)
                                        {
                                          advance(&cptr, &remembered_chars, buffer);
                                          pos++;
                                          if (pos >= len)
                                            return len;

                                          if ((*cptr == '\\') && (esc_buffer[state-1] == ESC))
                                            break;

                                          esc_buffer[state] = *cptr;
                                          state++;
                                        }
                          break;

                          default :   /* escape sequence */
                                      while ((*cptr < '0') || (*cptr > '~'))
                                        {
                                          if (*cptr == ESC)
                                            goto restart_ESC;
                                          if ((*cptr == CAN) || (*cptr == SUB))
                                            goto quit_ESC;
                                          if (*cptr == BS)
                                            state -= 2;

                                          advance(&cptr, &remembered_chars, buffer);
                                          pos++;
                                          if (pos >= len)
                                            return len;

                                          esc_buffer[state] = *cptr;
                                          state++;
                                        }
                                      esc_sequence();
                          break;
                        }
quit_ESC:
          break;

          case ENQ :  /* ignore */
          break;

          case HT :   /* go to next tab */
#ifdef DEBUG_VT220
                      fprintf(stderr,"[%c]", *cptr); fflush(stderr);
#endif
                      xpos++;
                      if (!dblw[ypos])
                        while ((xpos < screen_width) && !tabs[xpos])
                          xpos++;
                      else
                        while ((xpos < screen_width/2) && !tabs[xpos])
                          xpos++;
                      moveto(0, xpos, ypos);
          break;

          case BEL :  /* beep */
                      beep();
          break;

          case VT :   /* simulate new line */
          case FF :
          case LF :
#ifdef DEBUG_VT220
                      fprintf(stderr,"[\\%i]\n", (int) *cptr); fflush(stderr);
#endif
                      if (crlf)
                        screen_write(CR);
                      screen_write(LF);
          break;

          case SO :   /* ^N - select character set 1 */
          case SI :   /* ^O - select character set 0 */
#ifdef DEBUG_VT220
                      fprintf(stderr,"[%s]\n", *cptr==SO?"SO":"SI"); fflush(stderr);
#endif
                      charset = (*cptr) == SO;
          break;

          case NUL:   /* discard */
          break;

          default :   /* all other characters */

                      /* To speed up the display, write several characters
                       * at a time (no wrap or other special processing,
                       * default attribute)
                       */
                      if ((*cptr >= SP) && (*cptr <= '~')
                       && (xpos < screen_width) && (pos < len))
                        {
                          x    = xpos;
                          bptr = buf;

                          while ((*cptr >= SP) && (*cptr <= '~')
                              && (x < screen_width) && (pos < len))
                            {
                              *bptr = *cptr;

                              advance(&cptr, &remembered_chars, buffer);
                              pos++;
                              bptr++;
                              x++;
                            }
                          *bptr = 0;

#ifdef DEBUG_VT220
                          fprintf(stderr,"%s", buf); fflush(stderr);
#endif

                          screen_fastwrite(x-xpos, buf);

                          continue; /* cptr has been advanced already */
                        }
                        else        /* normal screen write */
                          {
#ifdef DEBUG_VT220
                            fprintf(stderr,"%c", *cptr); fflush(stderr);
#endif
                            screen_write(*cptr);
                          }
          break;
        }

      advance(&cptr, &remembered_chars, buffer);
      pos++;
    }

  state = 0;
  return len;
}


/*
 * Translate PC scan codes to VT220 ESC sequences
 *
 * This function discards characters if the buffer would overflow,
 * because input comes from a terminal and the user can be expected
 * not to type 32768 characters at a time.
 *
 * The F* function keys return Linux values. Shift-F* returns the
 * standard key code.
 *
 * This function MUST work correctly without prior initialization
 * by vt220_reset().
 */
void vt220_translate(char *buffer, int *len)
{
  char tmp[32768];
  unsigned char *sptr, *dptr;
  int left;

  memcpy(tmp, buffer, *len);

  sptr = tmp;
  dptr = buffer;
  left = *len;
  *len = 0;

  while (left > 0)
    {
      if (*sptr == 0)   /* found PC key scan code */
        {
          char *sequence;

          sptr++;
          left--;

          switch (*sptr)
            {
              case K_UP:
                        sequence = appl_mode ? "\eOA" : "\e[A";
              break;

              case K_DOWN:
                        sequence = appl_mode ? "\eOB" : "\e[B";
              break;

              case K_RIGHT:
                        sequence = appl_mode ? "\eOC" : "\e[C";
              break;

              case K_LEFT:
                        sequence = appl_mode ? "\eOD" : "\e[D";
              break;

              case K_SHIFT_F1:
                        sequence = "\eOP";
              break;

              case K_SHIFT_F2:
                        sequence = "\eOQ";
              break;

              case K_SHIFT_F3:
                        sequence = "\eOR";
              break;

              case K_SHIFT_F4:
                        sequence = "\eOS";
              break;

              case K_F1:
                        sequence = "\e[[A";
              break;

              case K_F2:
                        sequence = "\e[[B";
              break;

              case K_F3:
                        sequence = "\e[[C";
              break;

              case K_F4:
                        sequence = "\e[[D";
              break;

              case K_F5:
                        sequence = "\e[[E";
              break;

              case K_F6:
                        sequence = "\e[17~";
              break;

              case K_F7:
                        sequence = "\e[18~";
              break;

              case K_F8:
                        sequence = "\e[19~";
              break;

              case K_F9:
                        sequence = "\e[20~";
              break;

              case K_F10:
                        sequence = "\e[21~";
              break;

              case K_F11:
                        sequence = "\e[23~";
              break;

              case K_F12:
                        sequence = "\e[24~";
              break;

              case K_HOME:
                        sequence = "\e[1~";
              break;

              case K_INS:
                        sequence = "\e[2~";
              break;

              case K_DEL:
                        sequence = "\e[3~";
              break;

              case K_END:
                        sequence = "\e[4~";
              break;

              case K_PAGEUP:
                        sequence = "\e[5~";
              break;

              case K_PAGEDOWN:
                        sequence = "\e[6~";
              break;

              case K_CTRL_SPACE: /* generate 0 char */
                        *dptr = 0;
                        (*len)++;
                        if (*len == sizeof(tmp))
                          return;
                        dptr++;
                        sequence = "";
              break;

              default:  /* ignore all others */
                        sequence = "";
              break;
            }

          while (*sequence)
            {
              *dptr = *sequence;
              (*len)++;
              if (*len == sizeof(tmp)) /* ungraceful overflow handling */
                return;
              dptr++;
              sequence++;
            }
        }
      else              /* normal character, just append it */
        {
          *dptr = *sptr;
          (*len)++;
          if (*len == sizeof(tmp))  /* ungraceful overflow handling */
            return;
          dptr++;
        }

      sptr++;
      left--;
    }
}


void vt220_viogetmode(int *columns, int *rows, int *pelcols, int *pelrows)
{
  VIOMODEINFO ModeData;

  ModeData.cb = sizeof(ModeData);

  if (VioGetMode(&ModeData, 0) != 0)
    memset(&ModeData, 0, sizeof(ModeData));

  *columns = ModeData.col;
  *rows    = ModeData.row;
  *pelcols = ModeData.hres;
  *pelrows = ModeData.vres;
}


void vt220_viosetmode(int columns, int rows)
{
  VIOMODEINFO ModeData;

  ModeData.cb = sizeof(ModeData);

  if (VioGetMode(&ModeData, 0) != 0)
    return;

  ModeData.col = columns;
  ModeData.row = rows;
  VioSetMode(&ModeData, 0);
}

